/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_CACHE_H__
#define __WFS_CACHE_H__ 1

#include <wfs_file.h>
#include <wfs_protocol.h>
#include <string>
#include <vector>

typedef struct wfs_buf_write {
  unsigned long long offset;
  bool truncate;
  std::string buf;
} wfs_buf_write;

class wfs_cache {

 public:

  virtual ~wfs_cache() {}

  virtual int num_buf_writes( wfs_file_handle fh ) = 0;

  virtual bool read_md( wfs_file_handle fh, 
			wfs_protocol::fileinfo &finfo ) = 0;
  virtual bool read_any_md( wfs_file_handle fh, 
			    wfs_protocol::fileinfo &finfo ) = 0;
  virtual void store_md( wfs_file_handle fh, 
			 wfs_protocol::fileinfo finfo ) = 0;
  virtual void clear_md( wfs_file_handle fh, wfs_version version ) = 0;

  virtual bool read_block( wfs_file_handle fh, wfs_version version, 
			   unsigned int block_no, std::string &buf ) = 0;
  virtual bool read_block( wfs_file_handle fh, wfs_version version, 
			   unsigned int block_no ) = 0;
  virtual void store_block( wfs_file_handle fh, wfs_version version, 
			    unsigned int block_no, std::string buf ) = 0;
  // return false if there's not enough room to buffer this write
  virtual bool buf_write( wfs_file_handle fh, wfs_version version,
			  unsigned long long offset, bool truncate, 
			  const std::string &buf ) = 0;
  virtual void get_buf_writes( wfs_file_handle fh, 
			       std::vector<wfs_buf_write> &buf_writes ) = 0;
  virtual void apply_buf_writes( wfs_file_handle fh, wfs_version version,
				 unsigned int block_no,
				 std::string &buf ) = 0;

  // dir caching
  virtual bool dir_lookup( wfs_file_handle dh, std::string name,
			   wfs_file_handle &fh ) = 0;
  virtual void add_dir_entry( wfs_file_handle dh, 
			      std::string name, wfs_file_handle fh ) = 0;
  virtual bool readdir( wfs_file_handle dh, unsigned long long ind, 
			unsigned int n, 
			std::vector<wfs_storage::dirent> &e ) = 0;
  virtual void add_readdir_entry( wfs_file_handle dh, unsigned long long ind, 
				  std::vector<wfs_storage::dirent> e,
				  bool complete ) = 0;
  virtual void flush_dir( wfs_file_handle dh ) = 0;


};

#endif // __WFS_CACHE_H__
