/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


/*
 * configuration service tester
 * TODO: move me under tests.
 */
#include "rpc.h"
#include "vivaldi.h"
#include <arpa/inet.h>
#include <vector>
#include <wfs_client_manager.h>
#include "replication_policy.h"
#include "wfs_rsm_cclient.h"
#include "wfs_misc.h"
#include "slice_manager.h"

int nt = 10;
std::string dst;
const std::string me = "216.165.108.70:23001";
const host me_host("216.165.108.70", "23001");
int last_port = 0;
wfs_client_manager *clients;
const unsigned int EXPIRE = 30; // 1 minute.
typedef std::list<std::pair<wfs_slice_id,host> > reclist;

wfs_rsm_cclient **lc = new wfs_rsm_cclient * [nt];

int ct[256];
pthread_mutex_t count_mutex;

// convert a bit-string to a hex string for debug printouts.
std::string
hex(std::string s)
{
  char buf[64];
  unsigned int len = s.length();
  const char *p = s.c_str();
  unsigned int i;

  buf[0] = '\0';
  for(i = 0; i < len && i*2+1 < sizeof(buf); i++){
    sprintf(buf + (i * 2), "%02x", p[i] & 0xff);
  }

  return std::string(buf);
}

void
check_grant(std::string name)
{

}

void
check_release(std::string name)
{

}

inline host create_unique_host(const std::string ipaddr,const int tcount, int siteModulo,std::string & site){
  char* port = new char[6];
  char* csite = new char[6];
  snprintf(csite,6,"site%d",(tcount % siteModulo));
  snprintf(port,6,"%d",23000+tcount);
  port[5]='\0';
  csite[5]='\0';
  site = csite;
  return host(ipaddr.c_str(),port,site);
}

// RSM Client
/*
 * Simple Lock tests. Each Thread acquires a slice and renews it. Thread 0 releases the slice.
 */
void *
test1(void * arg)
{
  host prim("216.165.108.70","23001","siteA");
  host sec("10.0.0.1","23002","siteB");
  std::list<host> lst;
  lst.push_back(prim);lst.push_back(sec);
  rep_policy rp(1,rep_policy::STRICT,"siteA");
  wfs_slice_id sid;
  int tcount = *((int *)arg);

  for (int i = 0; i < nt; i++) {
    lc[i] = new wfs_rsm_cclient(dst,me_host,clients);
  }

  wfs_file_id fid = (((wfs_file_id) 1) << 58) +
    (((wfs_file_id) (tcount+1)) << 52);
  wfs_slice sl(fid);
  sid = sl.val();
  slice_assignment sa(sl,rp,lst,EXPIRE);

  lc[tcount]->acquire_slice(&sa,EXPIRE,true);

  printf("%s\n",(lc[tcount]->is_slice_locked(sid))?"LOCKED":"FREE");
  if(tcount == 0){
    sleep(30);
    lc[0]->release(&sa);
  }
  sleep(100);
  return NULL;
}

// Slice manager test.
/*
 * Simple Slice manager tests.
 * - Request file ID.
 * - Request another file ID from the same policy. Make sure that is from the same slice.
 * - Ask for a fid with an unknown primary site.
 * - Ask for a policy with a primary on a different site - should fail
 */
void * test2(void * arg)
{
  wfs_file_id fid,fid2,fid3,fid4,fid5;
  std::string site;
  host other;
  int tcount = *((int *)arg);
  if(tcount >= 10)
       return NULL;
  host _me = create_unique_host("216.165.108.70",tcount,10,site);
  slice_manager man(false,dst,_me,clients);
  rep_policy rp(1,rep_policy::STRICT,site);

  printf("*****THREAD:%d Request file_id for a simple policy\n",tcount);
  slice_manager::ret_code r = man.get_new_fid(rp,fid,other);
  assert(r == slice_manager::SM_SUCCESS);
  printf(" THREAD:%d Got file id:%x\n",tcount,fid);
  printf("*****THREAD:%d Request another file_id for the same policy\n",tcount);
  r = man.get_new_fid(rp,fid2,other,true);
  assert(r == slice_manager::SM_SUCCESS && ((fid & 0xFFF00000) == (fid2 & 0xFFF00000) ));
  printf(" THREAD:%d Got file id:%x\n",tcount,fid2);
  rp.prim_site = "SiteX";
  printf("*****THREAD:%d Request file_id for policy with unknown primary site\n",tcount);
  r = man.get_new_fid(rp,fid3,other);
  assert(r == slice_manager::SM_CANT_SATISFY);

  if(tcount != 0){
    rp.prim_site = "site0";
    sleep(5);
    man.sync_state();
    printf("*****THREAD:%d Request file_id for policy with known different primary site\n",tcount);
    r = man.get_new_fid(rp,fid4,other);
    assert(r == slice_manager::SM_TRYHOST);
    printf("*****THREAD:%d Manager suggested host:%s\n",tcount,other.str().c_str());

    // Test force_local_primary
    printf("*****THREAD:%d Request file_id for policy with known different primary site\n",tcount);
    r = man.get_new_fid(rp,fid4,other,true);
    assert(r == slice_manager::SM_TRYHOST);

    printf("*****THREAD:%d Request file_id for policy with different replication number\n",tcount);
    rp.prim_site = site;
    rp.rep_num = 2;
    r = man.get_new_fid(rp,fid5,other);
    assert(r == slice_manager::SM_SUCCESS);
    printf(" THREAD:%d Got file id:%x\n",tcount,fid5);

    std::vector<host> myhosts;
    r = man.get_all_active_hosts(myhosts);
    assert(r == slice_manager::SM_SUCCESS);
    printf("*****THREAD:%d Printing all hosts known to Manager.\n",tcount);
    for(std::vector<host>::iterator iter = myhosts.begin();iter != myhosts.end();++iter){
      printf("*****THREAD:%d\t Host:%s from site:%s\n",tcount,(*iter).str().c_str(),iter->site_id.c_str());
    }
    std::vector<wfs_slice_id> slices;
    r = man.get_maintenence_slices(slices);
    assert(r == slice_manager::SM_SUCCESS);
    printf("*****THREAD:%d Printing all slices this host participates in.\n",tcount);
      for(std::vector<wfs_slice_id>::iterator iter = slices.begin();iter != slices.end();++iter){
         printf("*****THREAD:%d\t Slice:%x\n",tcount,*iter);
    }
    man.print_all_known_slices();

  }
  while(1){};
  return NULL;
}

// Policy Repair test. and report host_down
/*
 * - Node 0 ask for a slice with repLevel 2 when only one host is known.
 * - Node 1 ask for a simple slice.
 * - Node 0 Should get host1 as a repair suggestions.
 * - Node 0 Should add host1 to the slice.
 * - Node 1 dies.
 * - Node 0 reports Thread 1 died.
 */
void *
test3(void * arg)
{
  std::string site;
  wfs_file_id fid;
  host other;
  int tcount = *((int *)arg);
  if(tcount >= 2)
    return NULL;
  host _me = create_unique_host("216.165.108.70",tcount,10,site);
  slice_manager man(false,dst,_me,clients);

  // Thread 0 asks for a policy with 2 replicas When only one host is known.
  rep_policy rp(1,rep_policy::STRICT,site);
  if(tcount == 0){
    rp.rep_num = 2;
    rp.site_num = 2;
    printf("*****THREAD:%d Request file_id\n",tcount);
    slice_manager::ret_code r = man.get_new_fid(rp,fid,other);
    assert(r == slice_manager::SM_SA_INC);
    printf(" THREAD:%d Got file id:%x\n",tcount,fid);
    printf("---THREAD:%d WITH BAD ASSIGNMENT\n",tcount);
    man.print_all_known_slices();
    sleep(10);

    man.sync_state();
    // Try and repair
    reclist recs;
    man.check_all_slices(recs);

    std::vector<host> dr;
    printf("*****THREAD:%d\t Needs to sync to:\n",tcount);
    for(reclist::iterator iter = recs.begin();iter != recs.end();++iter){
       printf("*****THREAD:%d\t Host:%s for slice:%016"PRIx64"\n",tcount,
           (iter->second).str().c_str(),iter->first);
       sleep(1);
       // Signal sync done
       bool answer;
       r = man.report_sync_complete(iter->first,iter->second,dr,answer);
       printf("Need to ask %s to lock? %s\n",(iter->second).str().c_str(),(answer?"YES":"NO"));
       assert(r == slice_manager::SM_SUCCESS);
       printf("*****THREAD:%d\t Reported Sync Complete for Host:%s for slice:%016"PRIx64"\n",tcount,
              (iter->second).str().c_str(),iter->first);

    }
    printf("---THREAD:%d AFTER REPAIR\n",tcount);
    man.print_all_known_slices();
    sleep(25);
    //const char *  oport = "23001";
    //host other("216.165.108.70",oport,"site1");
    host other = create_unique_host("216.165.108.70",1,10,site);
    printf("---THREAD:%d WITH HOST DOWN\n",tcount);
    man.print_all_known_slices();
    man.report_host_down(other);
    printf("---THREAD:%d HOST DOWN\n",tcount);
    man.print_all_known_slices();
    // Report host down.

  } else {

    // Other threads.
    sleep(2);
    printf("*****THREAD:%d Request file_id for a simple policy\n",tcount);
    slice_manager::ret_code r = man.get_new_fid(rp,fid,other);
    assert(r == slice_manager::SM_SUCCESS);
    printf(" THREAD:%d Got file id:%x\n",tcount,fid);
    // Thread dies.
  }

  return NULL;
}



// Create many different sites and ask for a policy with a high replication number.
void *
test4(void * arg)
{
  host other;
  std::string site;
  int tcount = *((int *)arg);
  if(tcount >= 10)
     return NULL;
  host _me = create_unique_host("216.165.108.70",tcount,3,site);
  slice_manager man(false,dst,_me,clients);
  rep_policy rp(1,rep_policy::STRICT,site);

  if(tcount == 0){
    rp.rep_num = 5;
    rp.site_num = 3;
    wfs_file_id fid;
    sleep(25);
    man.sync_state();
    printf("*****THREAD:%d Request file_id for a policy with many sites\n",tcount);
    slice_manager::ret_code r = man.get_new_fid(rp,fid,other);
    (void)r;
    //assert(r == slice_manager::SM_SUCCESS);
    printf(" THREAD:%d Got file id:%x\n",tcount,fid);
    man.print_all_known_slices();
    sleep(100);

  } else {
    sleep(1);
    printf("*****THREAD:%d Request file_id for a simple policy\n",tcount);
    wfs_file_id fid2;
    slice_manager::ret_code r = man.get_new_fid(rp,fid2,other,true);
    assert(r == slice_manager::SM_SUCCESS);
    printf(" THREAD:%d Got file id:%x\n",tcount,fid2);
    sleep(30);
    man.sync_state();
    man.print_all_known_slices();
    printf("Thread%d Exiting\n",tcount);
  }

  return NULL;
}

// Recovery Test. Thread 0 creates a slice with repNum2 and dies. Thread 1 take over the slice.
// Thread 0 wakes up.
void *
test5(void * arg)
{
  wfs_file_id fid;
  host other;
  int tcount = *((int *)arg);
  if(tcount >= 3)
     return NULL;
  std::string site;
  host _me = create_unique_host("216.165.108.70",tcount,10,site);
  slice_manager man(false,dst,_me,clients);

  rep_policy rp(1,rep_policy::STRICT,site);
  if(tcount == 0){
    sleep(5);
    man.sync_state();
    // This should assign a node on site1 as the secondary
    rp.rep_num = 2;
    rp.site_num = 2;
    printf("*****THREAD:%d Request file_id\n",tcount);
    slice_manager::ret_code r = man.get_new_fid(rp,fid,other);
    assert(r == slice_manager::SM_SUCCESS);
    printf(" THREAD:%d Got file id:%x\n",tcount,fid);
    printf("---THREAD:%d TWO HOSTS UP\n",tcount);
    man.print_all_known_slices();
    // Die
  } else if (tcount == 1){
    // Thread 1
    printf("*****THREAD:%d Request file_id for a simple policy\n",tcount);
    slice_manager::ret_code r = man.get_new_fid(rp,fid,other);
    assert(r == slice_manager::SM_SUCCESS);
    printf(" THREAD:%d Got file id:%x\n",tcount,fid);
    sleep(25);
    man.sync_state();
    // Host is down
    printf("---THREAD:%d HOST 0 WENT DOWN - BEFORE TAKING OVER SLICES\n",tcount);
    man.print_all_known_slices();
    host other("216.165.108.70","23000","site0");
    man.report_host_down(other);
    printf("---THREAD:%d HOST 0 WENT DOWN - AFTER TAKING OVER SLICES\n",tcount);
    man.print_all_known_slices();
    sleep(40);

    // Now should suggest host 0 as the host to sync with
    reclist recs;
    man.check_all_slices(recs);
    std::vector<host> dr;
    printf("*****THREAD:%d\t Needs to sync to:\n",tcount);
    for(reclist::iterator iter = recs.begin();iter != recs.end();++iter){
      printf("*****THREAD:%d\t Host:%s for slice:%016"PRIx64"\n",tcount,
          (iter->second).str().c_str(),iter->first);
       //Signal sync done
      bool answer = false;
      r = man.report_sync_complete(iter->first,iter->second,dr,answer);
      printf("Need to ask %s to lock? %s\n",(iter->second).str().c_str(),(answer?"YES":"NO"));
      assert(r == slice_manager::SM_SUCCESS);
      printf("*****THREAD:%d\t Reported Sync Complete for Host:%s for slice:%016"PRIx64"\n",tcount,
          (iter->second).str().c_str(),iter->first);
      assert(r == slice_manager::SM_SUCCESS);
      // See if slice was freed

    }
    printf("*****THREAD:%d\t After releasing slice \n",tcount);
    man.print_all_known_slices();

  } else {
    // Simulate thread 0 coming back to life.
    sleep(40);
    host _me1("216.165.108.70","23000","site0");
    slice_manager man2(false,dst,_me1,clients);
    man2.sync_state();
    // This should assign a node on site1 as the secondary
    rp.rep_num = 2;
    rp.site_num = 2;
    rp.prim_site = "site0";
    printf("BEFORE ASKING FOR NEW SLICE\n");
    man2.print_all_known_slices();
    printf("*****HOST 0 IS BACK UP. Request file_id\n");
    slice_manager::ret_code r = man2.get_new_fid(rp,fid,other,true);
    assert(r == slice_manager::SM_SUCCESS);
    printf(" THREAD:%d Got file id:%x\n",tcount,fid);
    man2.print_all_known_slices();
    while(1){};

  }

  return NULL;
}

void *
test6(void * arg)
{
  wfs_file_id fid;
  host other;
  int tcount = *((int *)arg);
  if(tcount > 0)
       return NULL;
  std::string site;
  host _me = create_unique_host("216.165.108.70",tcount,10,site);
  slice_manager man(false,dst,_me,clients);
  man.sync_state();
  man.print_all_known_slices();
  return NULL;
}

typedef void * (*thread_method)(void * arg);

thread_method testsArray[] = {&test1, &test2, &test3, &test4, &test5, &test6};

int
main(int argc, char *argv[])
{
  int r,loop;
  pthread_t th[nt];
  int test = 0;
  setvbuf(stdout, NULL, _IONBF, 0);
  setvbuf(stderr, NULL, _IONBF, 0);
  srandom(getpid());

  vivaldi viv;
  clients = new wfs_client_manager(&viv);

  //wfs_set_debug(1);

  if(argc < 2) {
    fprintf(stderr, "Usage: %s [host:]port [test] [test_arg]\n", argv[0]);
    exit(1);
  }
  dst = argv[1];

  if (argc > 2) {
    test = atoi(argv[2]);
    if(test < 1 || test > 6){
      printf("Test number must be between 1 and 5\n");
      exit(1);
    }
  }
  if(argc > 3){
    wfs_set_debug(atoi(argv[3]));
  }

  assert(pthread_mutex_init(&count_mutex, NULL) == 0);

  if(!test)
    loop = 5;
  else
    loop = 1;

  for(int j = 0; j < loop; ++ j){
    printf("Starting Test %d\n",(test)?test:j);
    for (int i = 0; i < nt; i++) {
      int *a = new int (i);
      r = pthread_create(&th[i], NULL, testsArray[(test)?test-1:j], (void *) a);
      assert (r == 0);
      sleep(1);
    }
    for (int i = 0; i < nt; i++) {
      pthread_join(th[i], NULL);
    }
  }

  printf ("%s: passed all tests successfully\n", argv[0]);

}

