/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


// updown: keep track of host liveness.
// maintains a set of hosts that might be up, and pings
// them to learn if they are actually up. gossips with
// other updowns.

#ifndef updown_h
#define updown_h

#include <host.h>
#include <rpc.h>
#include <vector>

struct host1 : host {
  host1(host hx, unsigned int xl) : host(hx), last(xl) { }
  host1() { }
  bool up( unsigned int timeout = 60 ) { 
    return((time(0) - last) < timeout);
  }
  unsigned int last;
};

class updown {
 private:
  rpcc *c;
  rpcs *s;
  host me;

  std::vector<host1> hosts;

  int handle(std::vector<host1>, int &dummy);
  void loop();
  int findhost(std::vector<host1>, host h);
  std::vector<host1> random_hosts();
  void send1(host);

 public:
  // start a thread to ping other hosts periodically.
  updown(rpcc *xc, rpcs *xs, host me);

  host myhost() { return me; }

  // anyone can call this to indicate that some host was known to
  // be contactable at UNIX time last_heard. or to add a host to
  // the set of hosts to ping periodically.
  void learn(host h, unsigned int last_heard);

  // get the list of hosts believed to be up.
  std::vector<host> get();
};

#endif
