/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef WCC_PROTOCOL_H_
#define WCC_PROTOCOL_H_

#include <rpc.h>
#include "slice_lock.h"

class wcc_protocol {
 public:
	 /*
	  * RPC Codes:
	  * 	NOENT: Client trying to release a non-existent lock.
	  * 	NOOWN: Client trying to release a lock that it does not own.
	  * 	STALE: Client's lock state is not up-to-date.
	  * 	RETRY: Lock is locked and under an unexpired lease.
	  */
  enum xxstatus { OK, RPCERR, NOENT, NOOWN, IOERR, STALE, RETRY };
  enum rpc_numbers {
    ping = 0x8001 | rpc_const::VIVALDI_MASK,
    //ping = 0x8001,
    fetch = 0x8002,
    acquire = 0x8003,
    release = 0x8004,
    renew = 0x8005,
    fetch_slices = 0x8006,
    update_auth_host_keys = 0x8007,
    update_auth_user_keys = 0x8008
  };

  struct ret {
      unsigned tryagain;
      std::string cur_state;
  };

  // Used to return all changes lock and the server's current viewstamp.
  struct fetch_ret {
    std::vector<slice_lock> slices;
    std::string auth_host_keys;
    std::map< std::string, std::string > auth_user_keys;
    viewstamp srvvs;
  };
};

inline unmarshall &
operator>>(unmarshall &u, wcc_protocol::ret &r)
{
  u >> r.tryagain;
  u >> r.cur_state;
  return u;
}

inline marshall &
operator<<(marshall &m, wcc_protocol::ret r)
{
  m << r.tryagain;
  m << r.cur_state;
  return m;
}

inline unmarshall &
operator>>(unmarshall &u, wcc_protocol::fetch_ret &r)
{
  u >> r.slices;
  u >> r.auth_host_keys;
  u >> r.auth_user_keys;
  u >> r.srvvs;
  return u;
}

inline marshall &
operator<<(marshall &m, wcc_protocol::fetch_ret r)
{
  m << r.slices;
  m << r.auth_host_keys;
  m << r.auth_user_keys;
  m << r.srvvs;
  return m;
}

// compiler freaks out when this is in the cc file, presumably because
// the definition of map unmarshalling is in the header?
inline void umap( unmarshall &u, std::map< std::string, std::string > &map ) {
  u >> map;
}

#endif /*WCC_PROTOCOL_H_*/
