/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#include "wfs_options.h"
#include "wfs_server.h"
#include <arpa/inet.h>
#include <slice_manager.h>
#include <rpc.h>

int init_and_run_wfs_fuse_client( wfs_options *opts, vivaldi *viv, 
				  slice_manager *slices, std::string progname,
				  wfs_client_manager *clients,
				  wfs_server *server = NULL );
wfs_server *init_wfs_server( wfs_options *opts, vivaldi *viv, 
			     slice_manager *slices,
			     wfs_client_manager *clients );

int main( int argc, char *argv[] ) {

  wfs_options opts( true, true );
  opts.get_options( argc, argv );
  opts.standard_init();

  vivaldi viv;
  wfs_client_manager clients(&viv);

  // cserver
  slice_manager slices( false, opts.cserver_master, opts.me_host, &clients,
			opts.ping_period, 2*opts.ping_period );

  wfs_server *ods = init_wfs_server( &opts, &viv, &slices, &clients );

  // give other nodes a chance to register their procs
  sleep(2);

  return init_and_run_wfs_fuse_client( &opts, &viv, &slices, argv[0],
				       &clients, ods );

}
