/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang,
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_OPTIONS_H__
#define __WFS_OPTIONS_H__ 1

#include <string>
#include <wfs_node.h>
#include <wfs_misc.h>

class wfs_options {

 private:
  bool _server;
  bool _client;
  WFSLogInit *_loginit;

 public:

  char *addr;
  unsigned int port;
  std::string site_id;
  char *port_char;
  char *cserver_master;
  char *dir;
  int debug_level;
  bool secure;
  std::string username;
  char *key_dir;
  int ping_period;
  int maint_timer;
  std::string ip;
  const char *cache_dir;
  char *client_port_char;
  int client_port;
  char *mountpoint;
  wfs_node me;
  host me_host;
  bool no_permissions;
  bool read_ahead;
  int prefetch_threads;
  std::string dbg_dir;

  wfs_options( bool server, bool client );
  ~wfs_options() { delete _loginit; };
  void usage( std::string progname );
  void get_options( int argc, char *argv[] );
  void standard_init();

};

#endif // __WFS_OPTIONS
