/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_MAINT_PROTOCOL_H__ 
#define __WFS_MAINT_PROTOCOL_H__ 1

#include <host.h>
#include <util/ident_util.h>

class wfs_maint_protocol {

 public:

  enum xxstatus { OK, RPCERR, NOENT, IOERR };
  typedef int status;

  enum rpc_numbers {
    md_exchange = 0x7001,
    pull_file_data,
    push_file_data,
    acquire_lock,
    md_exchange_slice
  };

  struct metadata {
    wfs_file_handle fh;
    wfs_version version;
    std::map<std::string, wfs_permission> permissions;
    unsigned long long checksum; // used for dirs
  };

  struct file_data {
    std::string data;
    std::string perm_cues;
    wfs_version version;
    int mode;
    std::map<std::string, wfs_permission> permissions;
    unsigned long atime;
    unsigned long mtime;
    int num_links;
  };

};

inline unmarshall &
operator>>(unmarshall &u, wfs_maint_protocol::metadata &e)
{
  u >> e.fh;
  u >> e.version;
  u >> e.permissions;
  u >> e.checksum;
  return u;
}

inline marshall &
operator<<(marshall &m, wfs_maint_protocol::metadata e)
{
  m << e.fh;
  m << e.version;
  m << e.permissions;
  m << e.checksum;
  return m;
}

inline unmarshall &
operator>>(unmarshall &u, wfs_maint_protocol::file_data &e)
{
  u >> e.data;
  u >> e.perm_cues;
  u >> e.version;
  u >> e.permissions;
  u >> e.atime;
  u >> e.mtime;
  u >> e.num_links;
  return u;
}

inline marshall &
operator<<(marshall &m, wfs_maint_protocol::file_data e)
{
  m << e.data;
  m << e.perm_cues;
  m << e.version;
  m << e.permissions;
  m << e.atime;
  m << e.mtime;
  m << e.num_links;
  return m;
}

#endif //  __WFS_MAINT_PROTOCOL_H__ 
