/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#include <node_list_simple.h>
#include <chash.h>
#include <stdio.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <stdlib.h>

wfs_node new_rand_node() {
  wfs_node new_node;
  new_node.id = rand_node_id();
  new_node.addr.sin_family = AF_INET;
  new_node.addr.sin_port = htons(3490);
  inet_aton("18.26.4.238", &new_node.addr.sin_addr);
  printf( "Made node %020"PRId64"\n", new_node.id );
  return new_node;
}

int main(int argc, char *argv[]) {

  srandom(time(NULL));

  int seed = random();
  if( argc > 1 ) {
    seed = atoi(argv[1]);
  }
  srandom(seed);
  printf( "Random seed = %d\n", seed );

  wfs_node me = new_rand_node();
  node_list *nl = new node_list_simple( me );

  for( uint i = 0; i < 10; i++ ) {
    wfs_node n = new_rand_node();
    nl->add_node(n);
  }

  wfs_node_id r = rand_node_id();
  wfs_node succ = nl->successor(r);
  printf( "Found succ %020"PRId64" to id %020"PRId64"\n", succ.id, r );
  wfs_node pred = nl->predecessor(r);
  printf( "Found pred %020"PRId64" to id %020"PRId64"\n", pred.id, r );

}
