/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang,
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_MISC_H__
#define __WFS_MISC_H__ 1

#include <arpa/inet.h>
#include <string>
#include <stdarg.h>
#include <pthread.h>
#include <map>
/*
 * Allow all files that use logging to still get the definitions.
 */
#include "logging/wfs_log.h"

void wfs_thread_id_init( int32_t mask = 0 );
int32_t wfs_set_thread_id( int32_t id );
int32_t wfs_get_thread_id();

/*
 * Used to add a wfs thread id printout to the external logging lib.
 */
class TidWFSLogAugmentor : public WFSLogAugmentor {
  public:
    ~TidWFSLogAugmentor() {};
    int addToOutput(FILE * fp);
};

float get_diff_ms( struct timeval start, struct timeval end );

const char *get_local_ip();
bool is_local_ip( in_addr addr );

#if defined(__linux__) || defined(__APPLE__)
unsigned long long md5_to_long( unsigned char *md5_result );
#else
unsigned long long md5_to_long( char *md5_result );
#endif

void copy_file(const std::string &from, const std::string &to,
	       bool async = false);

extern char default_keydir[];

void wfs_set_key_dir(const char *dir);
bool wfs_public_key_identifies_user(const char *username,
				    const char *public_key);
const char *wfs_get_private_key(const char *username);
const char *wfs_get_authorized_hosts_file();
const char *wfs_get_authorized_keys_dir();
void auth_update_hosts_file( const std::string hosts_file );
void auth_update_users_dir( const std::map< std::string, std::string > users_map );
std::string auth_hosts_file_string();
std::map<std::string, std::string> auth_user_keys_map();

#endif // __WFS_MISC_H__
